<?php

use AcyMailing\Libraries\acymPlugin;

class plgAcymCouryeah extends acymPlugin
{
    const SENDING_METHOD_ID = 'couryeah';
    const SENDING_METHOD_NAME = 'AcyMailing sending service';
    const SENDING_METHOD_API_URL = 'https://acymailer.jetpulp.work/api/';
    const CREDITS_RELOAD_DELAY = 900;

    public function __construct()
    {
        parent::__construct();
        $this->pluginDescription->name = self::SENDING_METHOD_NAME;
    }

    protected function callApiSendingMethod($url, $data = [], $headers = [], $type = 'GET', $authentication = [], $dataDecoded = false)
    {
        $url = plgAcymCouryeah::SENDING_METHOD_API_URL.$url;
        $defaultHeaders = [
            'accept' => 'application/json',
            'content-type' => 'application/json',
            'API-KEY' => $this->config->get(plgAcymCouryeah::SENDING_METHOD_ID.'_apikey', ''),
            'Version' => $this->config->get('version'),
        ];

        foreach ($defaultHeaders as $key => $value) {
            if (!isset($headers[$key])) {
                $headers[$key] = $value;
            }
        }

        $result = parent::callApiSendingMethod($url, $data, $headers, $type, $authentication, $dataDecoded);
        if (!empty($result['error_curl'])) {
            $result['message'] = $result['error_curl'];
        }

        return $result;
    }

    public function onAcymGetSendingMethods(&$data, $isMailer = false)
    {
        $data['sendingMethods'][self::SENDING_METHOD_ID] = [
            'name' => $this->pluginDescription->name,
            'image' => ACYM_IMAGES.'mailers/'.self::SENDING_METHOD_ID.'.png',
            'recommanded' => true,
            'image_class' => '',
        ];
    }

    public function onAcymGetSendingMethodsHtmlSetting(&$data)
    {
        $domains = $this->checkDomain();
        $domainsValidated = [];
        $domainsWaiting = [];
        $domainsFailed = [];
        if (!empty($domains)) {
            $domainsValidated = array_filter($domains, function ($domain) {
                return $domain['status'] === 'SUCCESS';
            });
            $domainsWaiting = array_filter($domains, function ($domain) {
                return $domain['status'] !== 'SUCCESS' && $domain['status'] !== 'FAILED';
            });
            $domainsFailed = array_filter($domains, function ($domain) {
                return $domain['status'] === 'FAILED';
            });
        }

        $unverifiedDomains = $this->getDomainsUnVerified();

        ob_start();
        ?>
		<div class="send_settings cell grid-x acym_vcenter" id="<?php echo self::SENDING_METHOD_ID; ?>_settings">
			<div class="cell grid-x acym_vcenter acym__sending__methods__one__settings">
				<label for="<?php echo self::SENDING_METHOD_ID; ?>_apikey" class="cell"><?php echo acym_translation('ACYM_API_KEY'); ?></label>
				<div class="cell grid-x">
					<input id="<?php echo self::SENDING_METHOD_ID; ?>_apikey"
						   class="cell medium-6 xlarge-8"
						   type="text"
						   name="config[<?php echo self::SENDING_METHOD_ID; ?>_apikey]"
						   value="<?php echo acym_escape($this->config->get(self::SENDING_METHOD_ID.'_apikey')); ?>">
					<div class="cell grid-x xlarge-3 padding-left-1 acym__sending__methods__credentials__test">
						<button type="button"
								sending-method-id="<?php echo self::SENDING_METHOD_ID; ?>"
								class="acym__configuration__sending__method-test cell shrink button button-secondary">
                            <?php echo acym_translation('ACYM_TEST_CREDENTIALS'); ?>
						</button>
						<span class="acym__configuration__sending__method-icon cell shrink margin-left-1 acym_vcenter"></span>
						<span class="acym__configuration__sending__method-test__message cell shrink margin-left-1 acym_vcenter"></span>
					</div>
				</div>
			</div>
			<div class="cell grid-x acym__sending__methods__one__settings margin-top-1">
				<div class="cell medium-5 large-6 xlarge-3 walkthrough__responsive">
					<label>
                        <?php echo acym_translation('ACYM_DOMAINS_VALIDATED'); ?>
					</label>
					<div id="acym__sending__methods__validate__domains" class="grid-x">
                        <?php
                        if (!empty($domainsValidated)) {
                            foreach ($domainsValidated as $domain) {
                                ?>
								<div class="cell grid-x acym__couryeah__domain_containers acym__sending__methods__container__oneVerifiedDomain">
									<p class="acym__sending__methods__domain acym__sending__methods__verifiedDomain medium-6">
                                        <?php echo $domain['domain']; ?>
									</p>
									<i class="margin-left-1 margin-right-1 cursor-pointer acym_vcenter acymicon-trash-o acym__color__red acym__sending__methods__validate_domain-delete"></i>
								</div>
                                <?php
                            }
                        } else {
                            ?>
							<p class="medium-6"><?php echo acym_translation('ACYM_NO_DOMAINS') ?></p>
                        <?php } ?>
					</div>
				</div>
				<div class="cell medium-5 large-6 xlarge-3 walkthrough__responsive">
					<label>
                        <?php echo acym_translation('ACYM_DOMAINS_WAITING_FOR_APPROVAL'); ?>
					</label>
					<div id="acym__sending__methods__approval__domains" class="grid-x">
                        <?php
                        if (!empty($domainsWaiting)) {
                            foreach ($domainsWaiting as $oneDomain) {
                                ?>
								<div class="cell acym__sending__methods__container__oneUnverifiedDomain acym__couryeah__domain_containers grid-x">
									<p class="acym__sending__methods__domain acym__sending__methods__unverifiedDomain medium-4"
									   data-acym-cname="<?php echo acym_escape($oneDomain['CnameRecords']); ?>">
                                        <?php echo $oneDomain['domain']; ?>
									</p>
									<i class="medium-2 acymicon-binoculars acym_vcenter acym__sending__methods__unverifiedDomain-icon cursor-pointer"></i>
								</div>
                                <?php
                            }
                        } else {
                            ?>
							<p class="medium-6"><?php echo acym_translation('ACYM_NO_DOMAINS') ?></p>
                        <?php } ?>
					</div>
				</div>
				<div class="cell medium-5 large-6 xlarge-3 walkthrough__responsive">
					<label>
                        <?php echo acym_translation('ACYM_DOMAINS_FAILED_APROVAL'); ?>
					</label>
					<div id="acym__sending__methods__approval__domains" class="grid-x">
                        <?php
                        if (!empty($domainsFailed)) {
                            foreach ($domainsFailed as $oneDomain) {
                                ?>
								<div class="cell acym__sending__methods__container__oneUnverifiedDomain acym__couryeah__domain_containers grid-x">
									<p class="acym__sending__methods__domain acym__sending__methods__unverifiedDomain medium-4"
									   data-acym-cname="<?php echo acym_escape($oneDomain['CnameRecords']); ?>">
                                        <?php echo $oneDomain['domain']; ?>
									</p>
									<i class="margin-left-1 margin-right-1 cursor-pointer acym_vcenter acymicon-trash-o acym__color__red acym__sending__methods__validate_domain-delete"></i>
								</div>
                                <?php
                            }
                        } else {
                            ?>
							<p class="medium-6"><?php echo acym_translation('ACYM_NO_DOMAINS') ?></p>
                        <?php } ?>
					</div>
				</div>
			</div>
			<div class="cell grid-x acym_vcenter acym__sending__methods__one__settings padding-top-1">
				<div class="cell grid-x">
					<div>
						<input id="<?php echo self::SENDING_METHOD_ID; ?>_domain"
							   class="cell medium-6 large-4 xlarge-3"
							   type="text"
							   autocomplete="off"
							   value="">
						<span id="<?php echo self::SENDING_METHOD_ID; ?>_domain_error" class="medium-6 large-4 xlarge-3"></span>
                        <?php if (!empty($unverifiedDomains)) { ?>
							<span id="acym__couryeah__unverifiedDomains">
                        <?php foreach ($unverifiedDomains as $oneDomain) { ?>
							<span class="acym__couryeah__oneSuggestion"><?php echo $oneDomain; ?></span>
                        <?php } ?>
						</span>
                        <?php } ?>
					</div>
					<div id="acym__configuration__sending__method_addDomain_submit" class="cell grid-x medium-6 large-8 padding-left-1">
						<button type="button"
								id="acym__configuration__sending__method-addDomain"
								class="cell shrink button button-secondary">
                            <?php echo acym_translation('ACYM_ADD_A_DOMAIN'); ?>
						</button>
						<i class="margin-left-1 acym_vcenter acymicon-circle-o-notch acymicon-spin is-hidden" id="acym__configuration__sending__method_add_domain-wait"></i>
					</div>
				</div>
			</div>
			<div id="acym__configuration__sending__method__cnameTable__container" class="cell" style="display: none">
				<p><?php echo acym_translation('ACYM_DNS_ENTRIES') ?></p>
				<div class="grid-x acym__listing" id="acym__listing_cnames">
					<div class="grid-x cell acym__listing__header">
						<div id="acym__configuration__sending__method__cnameTable grid-x acym_listing"
							 class="grid-x medium-auto small-11 cell acym__listing__header__title__container">
							<div class="cell medium-4 small-7 xlarge-6 acym__listing__header__title"><?php echo acym_translation('ACYM_NAME'); ?></div>
							<div class="cell medium-4 small-7 xlarge-6 acym__listing__header__title"><?php echo acym_translation('ACYM_VALUE'); ?></div>
						</div>
					</div>
					<div class="grid-x cell acym__listing__row">
						<div class="grid-x medium-auto small-11 cell acym__listing__title__container">
							<div class="cell medium-4 small-7 xlarge-6 cname-name">
							</div>
							<div class="cellmedium-4 small-7 xlarge-6 cname-value">
							</div>
						</div>
					</div>
					<div class="grid-x cell acym__listing__row">
						<div class="grid-x medium-auto small-11 cell acym__listing__title__container">
							<div class="cell medium-4 small-7 xlarge-6 cname-name">
							</div>
							<div class="cellmedium-4 small-7 xlarge-6 cname-value">
							</div>
						</div>
					</div>
					<div class="grid-x cell acym__listing__row">
						<div class="grid-x medium-auto small-11 cell acym__listing__title__container">
							<div class="cell medium-4 small-7 xlarge-6 cname-name">
							</div>
							<div class="cellmedium-4 small-7 xlarge-6 cname-value">
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>
        <?php
        $data['sendingMethodsHtmlSettings'][self::SENDING_METHOD_ID] = ob_get_clean();
    }

    public function onAcymTestCredentialSendingMethod($sendingMethod, $credentials)
    {
        if ($sendingMethod !== self::SENDING_METHOD_ID) return;

        if (empty($credentials[self::SENDING_METHOD_ID.'_apikey'])) {
            acym_sendAjaxResponse(acym_translation('ACYM_MISSING_API_KEY'), [], false);
        }

        $response = $this->callApiSendingMethod(
            'get_credits',
            [],
            [
                'API-KEY' => $credentials[self::SENDING_METHOD_ID.'_apikey'],
            ]
        );

        if (empty($response)) {
            $errorMsg = acym_translation('ACYM_NO_ANSWER');
            acym_sendAjaxResponse(acym_translationSprintf('ACYM_ERROR_OCCURRED_WHILE_CALLING_API', $errorMsg), [], false);
        } elseif (!empty($response['error_curl'])) {
            acym_sendAjaxResponse(acym_translationSprintf('ACYM_ERROR_OCCURRED_WHILE_CALLING_API', $response['error_curl']), [], false);
        } elseif (!empty($response['message']) && $response['message'] == 'Invalid private key') {
            acym_sendAjaxResponse(acym_translation('ACYM_AUTHENTICATION_FAILS_WITH_API_KEY'), [], false);
        } elseif (!empty($response['message'])) {
            acym_sendAjaxResponse($response['message'], [], false);
        } else {
            acym_sendAjaxResponse(acym_translation('ACYM_API_KEY_CORRECT'));
        }
    }

    public function onAcymProcessQueueExternalSendingCampaign(&$externalSending, $transactional = false)
    {
        if ($this->config->get('mailer_method') == self::SENDING_METHOD_ID) $externalSending = false;
    }

    public function onAcymSendEmail(&$response, $mailerHelper, $to, $from, $replyTo, $bcc = [], $attachments = [])
    {
        if ($mailerHelper->externalMailer != self::SENDING_METHOD_ID) return;

        $apikey = $this->config->get(self::SENDING_METHOD_ID.'_apikey');
        if (empty($apikey)) {
            $response['error'] = true;
            $response['message'] = acym_translation('ACYM_MISSING_API_KEY');

            return;
        }

        $domainsUsed = [
            acym_getDomain($from['email']),
            acym_getDomain($replyTo['email']),
        ];
        $bounceAddress = $this->config->get('bounce_email');
        if (!empty($bounceAddress)) {
            $domainsUsed[] = acym_getDomain($bounceAddress);
        }

        $domainsUsed = array_unique($domainsUsed);

        $verifiedDomains = $this->getVerifiedDomains();

        $unverifiedDomains = array_diff($domainsUsed, $verifiedDomains);
        if (!empty($unverifiedDomains)) {
            $response['error'] = true;
            $response['message'] = acym_translationSprintf('ACYM_UNVERIFIED_DOMAINS_PREVENTING_EMAILS_FROM_BEING_SENT_X', implode(', ', $unverifiedDomains));

            return;
        }

        $responseMailer = $this->callApiSendingMethod(
            'send',
            [
                'email' => $mailerHelper->getSentMIMEMessage(),
                'domainsUsed' => $domainsUsed,
            ],
            [],
            'POST'
        );

        if (!empty($responseMailer['error_curl'])) {
            $response['error'] = true;
            $response['message'] = acym_translationSprintf('ACYM_ERROR_OCCURRED_WHILE_CALLING_API', $responseMailer['error_curl']);
        } elseif ($this->responseCode !== 200) {
            $response['error'] = true;
            $response['message'] = $responseMailer['message'];

            if ($response['message'] === 'No credits left') {
                $response['message'] = acym_translation('ACYM_NOT_ENOUGH_CREDITS').' : ';
                $response['message'] .= '<a target="_blank" class="acym__color__blue" href="'.ACYM_ACYMAILLING_WEBSITE.'">';
                $response['message'] .= acym_translation('ACYM_GET_MORE_CREDITS');
                $response['message'] .= '</a>';
            }
        } else {
            $response['error'] = false;
        }
    }

    private function checkDomain()
    {
        $domains = json_decode($this->config->get(self::SENDING_METHOD_ID.'_domains', '[]'), true);

        if (empty($domains)) return [];

        if ($this->config->get('mailer_method') !== self::SENDING_METHOD_ID) return $domains;

        $remainingDomains = $this->checkDomainsDNS($domains);
        if (empty($remainingDomains)) return $domains;

        $prepareDomains = array_map(
            function ($domain) {
                return $domain['domain'];
            },
            $remainingDomains
        );

        $responseApi = $this->callApiSendingMethod(
            'getDomainStatus',
            ['domains' => $prepareDomains],
            [],
            'POST'
        );

        if (empty($responseApi)) {
            return $domains;
        }

        if (!$this->checkSuccessCode($this->responseCode)) {
            acym_enqueueMessage(acym_translation('ACYM_ERROR_ON_CALL_ACYBA_WEBSITE').': '.$responseApi['message'], 'error');

            return $domains;
        }

        foreach ($domains as $key => $oneDomain) {
            $domainName = $oneDomain['domain'];
            if (!is_array($responseApi['data'][$domainName])) {
                acym_enqueueMessage(acym_translation('ACYM_ERROR_ON_CALL_ACYBA_WEBSITE').': '.$responseApi['data'][$domainName].': '.$domainName, 'error');
                continue;
            }
            $domains[$key]['status'] = $responseApi['data'][$domainName]['status'];
        }

        $this->config->save([self::SENDING_METHOD_ID.'_domains' => json_encode($domains)]);

        return $domains;
    }

    public function checkDomainsDNS($domains)
    {
        $remainingDomains = [];
        foreach ($domains as $oneDomain) {
            if (empty($oneDomain['CnameRecords'])) {
                $remainingDomains[] = $oneDomain;
                continue;
            }
            if ($oneDomain['status'] == 'SUCCESS') {
                continue;
            }
            $shouldCheckDomain = true;
            foreach ($oneDomain['CnameRecords'] as $oneRecord) {
                $dnsEntries = @dns_get_record($oneRecord['name'], DNS_CNAME);
                if (empty($dnsEntries)) {
                    $shouldCheckDomain = false;
                    break; // no need to check this domain
                }
                $entryFound = false;
                if (!is_array($dnsEntries)) {
                    $dnsEntries = [$dnsEntries];
                }
                foreach ($dnsEntries as $oneEntry) {
                    if ($oneEntry['type'] !== 'CNAME' || $oneEntry['host'] !== $oneRecord['name']) {
                        continue;
                    }
                    if ($oneEntry['target'] === $oneRecord['value']) {
                        $entryFound = true;
                        break;
                    } else {
                        acym_enqueueMessage(
                            acym_translation_sprintf('ACYM_DOMAIN_CHECK_BAD_SYNTAX', $oneRecord['name'], $oneDomain['domain']),
                            'error'
                        );
                    }
                }
                if (!$entryFound) {
                    $shouldCheckDomain = false;
                }
            }
            if ($shouldCheckDomain) {
                $remainingDomains[] = $oneDomain;
            }
        }

        return $remainingDomains;
    }

    public function onAcymDeleteDomain()
    {
        $sendingMethod = acym_getVar('string', 'sendingMethod', '');
        $oneDomain = acym_getVar('string', 'oneDomain', '');

        if ($sendingMethod != self::SENDING_METHOD_ID) return;
        if (empty($oneDomain)) return;

        $apikey = $this->config->get(self::SENDING_METHOD_ID.'_apikey');
        if (empty($apikey)) {
            acym_sendAjaxResponse(acym_translation('ACYM_MISSING_API_KEY'), [], false);
        }

        $responseApi = $this->callApiSendingMethod(
            'deleteDomainIdentity',
            ['domain' => $oneDomain, 'siteUrl' => acym_baseURI()],
            [],
            'POST'
        );

        if (!empty($responseApi['error_curl'])) {
            $message = acym_translationSprintf('ACYM_ERROR_OCCURRED_WHILE_CALLING_API', $responseApi['error_curl']);
            acym_sendAjaxResponse($message, [], false);
        } elseif (!empty($responseApi['message']) && $responseApi['message'] == 'Invalid private key') {
            acym_sendAjaxResponse(acym_translation('ACYM_AUTHENTICATION_FAILS_WITH_API_KEY'), [], false);
        } elseif (!$this->checkSuccessCode($this->responseCode)) {
            $message = acym_translation('ACYM_ERROR_ON_CALL_ACYBA_WEBSITE').': '.$responseApi['message'];
            acym_sendAjaxResponse($message, [], false);
        } else {
            $field = self::SENDING_METHOD_ID.'_domains';
            $domains = json_decode($this->config->get($field, '[]'), true);
            $domains = array_filter($domains, function ($domain) use ($oneDomain) {
                return $domain['domain'] != $oneDomain;
            });

            $newConfig = new \stdClass();
            $newConfig->$field = json_encode($domains);
            $this->config->save($newConfig);

            acym_sendAjaxResponse();
        }
    }

    public function ajaxAddDomain()
    {
        $sendingMethod = acym_getVar('string', 'sendingMethod', '');
        $oneDomain = acym_getVar('string', 'oneDomain', '');

        if ($sendingMethod != self::SENDING_METHOD_ID) {
            acym_sendAjaxResponse(acym_translation('ACYM_COULD_NOT_FIND_SENDING_METHOD'), [], false);
        }

        if (empty($oneDomain)) {
            acym_sendAjaxResponse(acym_translation('ACYM_MISSING_DOMAIN'), [], false);
        }

        $apikey = $this->config->get(self::SENDING_METHOD_ID.'_apikey');
        if (empty($apikey)) {
            acym_sendAjaxResponse(acym_translation('ACYM_MISSING_API_KEY'), [], false);
        }

        $domainValidation = preg_match('/([\S\p{L}]+\.)+[\S\p{L}]{2,}/u', $oneDomain);
        if (!$domainValidation) {
            $message = acym_translationSprintf('ACYM_THE_DOMAIN_X_IS_NOT_VALID', $oneDomain);
            acym_sendAjaxResponse($message, ['type' => 'DOMAIN_VALIDATION'], false);
        }

        $field = self::SENDING_METHOD_ID.'_domains';
        $domains = $this->config->get($field, []);
        if (!empty($domains)) {
            $domains = json_decode($domains, true);
        }

        if (!empty($domains[$oneDomain])) {
            acym_sendAjaxResponse(acym_translation('ACYM_DOMAIN_ALREADY_SENT'), [], false);
        }

        $responseApi = $this->callApiSendingMethod(
            'createDomainIdentity',
            ['domain' => $oneDomain, 'siteUrl' => acym_baseURI()],
            [],
            'POST'
        );

        if (!empty($responseApi['error_curl'])) {
            $message = acym_translationSprintf('ACYM_ERROR_OCCURRED_WHILE_CALLING_API', $responseApi['error_curl']);
            acym_sendAjaxResponse($message, [], false);
        } elseif ($responseApi['message'] === 'Invalid API-KEY') {
            acym_sendAjaxResponse(acym_translation('ACYM_AUTHENTICATION_FAILS_WITH_API_KEY'), [], false);
        } elseif (!$this->checkSuccessCode($this->responseCode)) {
            $message = acym_translationSprintf($this->displayMessage($responseApi['message']), $oneDomain);
            acym_sendAjaxResponse($message, [], false);
        } elseif ($this->responseCode === 201) {
            $domains[$oneDomain] = [
                'domain' => $oneDomain,
                'CnameRecords' => $responseApi['cnameRecords'],
                'status' => 'PENDING',
            ];
            $this->config->save([$field => json_encode($domains)]);
            acym_sendAjaxResponse($responseApi['message'], ['cnameRecords' => $responseApi['cnameRecords']]);
        } else {
            $cnameRecords = $responseApi['data'][$oneDomain]['cnameRecords'];
            $status = $responseApi['data'][$oneDomain]['status'];
            $domains[$oneDomain] = [
                'domain' => $oneDomain,
                'CnameRecords' => $cnameRecords,
                'status' => $status,
            ];
            $this->config->save([$field => json_encode($domains)]);

            acym_sendAjaxResponse(
                $responseApi['message'],
                ['cnameRecords' => $cnameRecords, 'alreadyVerified' => $status === 'SUCCESS']
            );
        }
    }

    public function getDomainsUnVerified()
    {
        if ($this->config->get('mailer_method') !== self::SENDING_METHOD_ID) return [];

        $allDomains = [];

        $allDomains[] = acym_getDomain($this->config->get('from_email'));
        if ($this->config->get('from_as_replyto') === '0') {
            $allDomains[] = acym_getDomain($this->config->get('replyto_email'));
        }

        if ($this->config->get('multilingual') === '1') {
            $senderInfoTranslation = $this->config->get('sender_info_translation');
            if (!empty($senderInfoTranslation)) {
                $senderInfoTranslation = json_decode($senderInfoTranslation, true);
                foreach ($senderInfoTranslation as $oneSenderInfo) {
                    $allDomains[] = acym_getDomain($oneSenderInfo['from_email']);
                    $allDomains[] = acym_getDomain($oneSenderInfo['replyto_email']);
                }
            }
        }

        $allDomains[] = acym_getDomain($this->config->get('bounce_email'));
        $allDomains = array_filter(array_unique($allDomains));

        $verifiedDomains = $this->getVerifiedDomains();

        return array_diff($allDomains, $verifiedDomains);
    }

    private function getVerifiedDomains()
    {
        $verifiedDomains = [];
        $domainStatuses = json_decode($this->config->get(self::SENDING_METHOD_ID.'_domains', '[]'), true);
        if (!empty($domainStatuses)) {
            foreach ($domainStatuses as $domain => $info) {
                if ($info['status'] === 'SUCCESS') {
                    $verifiedDomains[] = $domain;
                }
            }
        }

        return $verifiedDomains;
    }

    public function onAcymDisplayPage()
    {
        $unverifiedDomains = $this->getDomainsUnVerified();

        if (!empty($unverifiedDomains)) {
            $message = acym_translationSprintf('ACYM_UNVERIFIED_DOMAINS_PREVENTING_EMAILS_FROM_BEING_SENT_X', implode(', ', $unverifiedDomains));
            acym_enqueueMessage($message, 'warning');
        }
    }

    private function displayMessage($message)
    {
        $correspondances = [
            'AlreadyExistsException' => 'ACYM_DOMAIN_X_ALREADY_EXIST',
            'TooManyRequestsException' => 'ACYM_DOMAIN_TO_MANY_REQUEST',
            'NotFoundException' => 'ACYM_DOMAIN_NOT_FOUND',
            'NotExistException' => 'ACYM_CANT_DELETE_DOMAIN',
        ];

        if (empty($message) || empty($correspondances[$message])) {
            return 'ACYM_DOMAIN_DEFAULT_ERROR';
        }

        return $correspondances[$message];
    }

    public function onAcymGetCreditRemainingSendingMethod(&$html, $reloading = false)
    {
        $sendingMethod = $this->config->get('mailer_method', '');
        if (empty($sendingMethod) || $sendingMethod != plgAcymCouryeah::SENDING_METHOD_ID) return;

        $apiKey = $this->config->get(plgAcymCouryeah::SENDING_METHOD_ID.'_apikey', '');
        if (empty($apiKey)) return;

        $credits = $this->getRemainingCredits($reloading);

        if (!isset($credits['remaining_credits'])) {
            if (empty($credits['message'])) {
                $html = acym_translation('ACYM_ERROR_CONTACT_ACYMAILING');
            } else {
                $html = acym_translationSprintf(
                    'ACYM_SENDING_METHOD_ERROR_WHILE_ACTION',
                    plgAcymCouryeah::SENDING_METHOD_NAME,
                    acym_translation('ACYM_GETTING_REMAINING_CREDITS'),
                    $credits['message']
                );
            }

            return;
        }

        $html = acym_translationSprintf(
            'ACYM_SENDING_METHOD_X_UNITY',
            plgAcymCouryeah::SENDING_METHOD_NAME,
            acym_tooltip(
                $credits['remaining_credits'],
                acym_translation('ACYM_GET_MORE_CREDITS'),
                'credits_remaining',
                '',
                ACYM_ACYMAILLING_WEBSITE
            ),
            '<span class="acym_not_bold">'.acym_translation('ACYM_CREDITS_REMAINING').'</span>'
        );
    }

    private function getRemainingCredits($forceReload = false)
    {
        $time = time();
        $lastCheck = $this->config->get(plgAcymCouryeah::SENDING_METHOD_ID.'_last_credits_check', 0);
        $lastDetails = $this->config->get(plgAcymCouryeah::SENDING_METHOD_ID.'_credits_details', []);

        if (!$forceReload && !empty($lastDetails) && $lastCheck > $time - plgAcymCouryeah::CREDITS_RELOAD_DELAY) {
            return json_decode($lastDetails, true);
        }

        $response = $this->callApiSendingMethod('get_credits');

        $this->config->save([
            plgAcymCouryeah::SENDING_METHOD_ID.'_credits_details' => json_encode($response),
            plgAcymCouryeah::SENDING_METHOD_ID.'_last_credits_check' => $time,
        ]);

        return $response;
    }

    public function onAcymCreditsLeft(&$creditsLeft)
    {
        $sendingMethod = $this->config->get('mailer_method', '');
        if (empty($sendingMethod) || $sendingMethod != plgAcymCouryeah::SENDING_METHOD_ID) return;

        $apiKey = $this->config->get(plgAcymCouryeah::SENDING_METHOD_ID.'_apikey', '');
        if (empty($apiKey)) {
            $creditsLeft = 0;

            return;
        }

        $credits = $this->getRemainingCredits(true);
        if (empty($credits['remaining_credits'])) {
            $creditsLeft = 0;
        } else {
            $creditsLeft = $credits['remaining_credits'];
        }
    }

    public function checkSuccessCode($responseCode)
    {
        $successCodes = [200, 201, 204];

        return in_array($responseCode, $successCodes);
    }
}
